// Exercise 12.6.
// This sample code is taken from my Secrets of the Borland C++ Masters
// book.  Published by Sams, 1992.  This program implements the
// line chart suggested for the exercise in Chapter 12, Exercise #6.
//
//
//  LINCHART.CPP
//
//  Demonstrate how to create a line chart.
//
//
#include <graphics.h>
#include <strstrea.h>
#include <iomanip.h>
#include <stdlib.h>
#include <string.h>
#include <conio.h>
#include <math.h>

typedef int BOOL;		// boolean

//
//	Global constants
//
const int maxDataValues = 20;
const int maxTitleLen = 128;
const int maxLabelLen = 64;

//
//	Function prototypes
//
void CalcMinAndMax(float &min, float &max, BOOL &fThousands,
		   float *pData, int nVals);
void DrawLineChart(char *pMainTitle, char *pxAxisTitle,
                   char *pyAxisTitle,
                   char  pLabels[][maxLabelLen],
		   float *pData, int nVals);
void DrawLines(int xLeft, int yTop, int xRight, int yBottom,
	       float minVal, float maxVal,
	       float *pData, int nVals,
	       int lineColor=WHITE,
	       int posPtColor=GREEN, int negPtColor=RED);
void DrawPoint(int x, int y, int color = WHITE, int size=8);
void DrawTitles(char *pMainTitle, char *pxAxisTitle,
		char *pyAxisTitle,
		int xLeft, int yTop,
		int xRight, int yBottom,
		int textColor = WHITE);
void DrawXAxisLabels(int xLeft, int xRight, int yBottom,
		int nVals, char pLabels[][maxLabelLen],
		int textColor = WHITE);
void DrawYAxisInfo(int xLeft, int yTop,
		 int xRight,int yBottom,
		 float minVal, float maxVal, int nDivs,
		 int lineColor = WHITE,
		 int textColor = WHITE);
int GetGraphInfo(char *pMainTitle, char *pxAxisTitle,
	       char *pyAxisTitle,
	       char  pLabels[][maxLabelLen],float *pData);
void InitGraphics(const char *pBGIPath);
void PromptAndWait(const char *pText, int textColor=WHITE);

//
//  Initialize the graphics system via auto detection.
//  Supply the path to the BGI files on your system.
//
void InitGraphics(const char *pBGIPath)
{
  //	  Request auto detection of graphics driver.
  int graphDriver = DETECT;
  int graphMode;
  detectgraph(&graphDriver, &graphMode);

  //  Initialize graphics system.
  //  Look for files in specified directory.
  initgraph(&graphDriver, &graphMode, pBGIPath);

  int graphError = graphresult();
  if(graphError != grOk) {
     cout << "Graphics error: "
          << grapherrormsg(graphError) << '\n';
     exit(1);
  }
}

//
// Display a prompt and wait for a keypress.
//
void PromptAndWait(const char *pText, int textColor)
{
  settextjustify(LEFT_TEXT, BOTTOM_TEXT);
  settextstyle(DEFAULT_FONT, HORIZ_DIR, 1);
  setcolor(textColor);
  outtextxy(10, getmaxy()-10, pText);
  getch();
}

//
//	Get the graph titles and data values.
//  Return the number of values read.
//
int GetGraphInfo(char *pMainTitle, char *pxAxisTitle,
		 char *pyAxisTitle,
		 char  pLabels[][maxLabelLen],
		 float *pData)
{
  clrscr();

  // Use cin.getline() rather than >> to allow whitespace.
  // Use getline() instead of get() to discard '\n'.
  cout << "Enter main graph title: " << endl;
  cin.getline(pMainTitle, maxTitleLen);
  cout << "Enter x-axis title: " << endl;
  cin.getline(pxAxisTitle, maxTitleLen);
  cout << "Enter y-axis title: " << endl;
  cin.getline(pyAxisTitle, maxTitleLen);

  // Get all the data
  // Note that multiple values on one line are okay
  cout << "Enter data values (99999 when done): " << endl;
  int nVals = 0;
  float val;
  while(1) {
    cin >> val;
    if(val == 99999)
	    break;
    pData[nVals++] = val;
  }

  // Discard any trailing characters
  cin.ignore(255, '\n');

  // Get a label for each piece of data
  for(int i=0; i<nVals; i++) {
    cout << "Enter label #" << i << ": " << endl;
    cin.getline(pLabels[i], maxLabelLen);
  }

  return nVals;
}

//
//  Find the minimum and maximum values in a dataset.
//  Set fThousands TRUE if the absolute value of any entry
//  is greater than 1000.
//
void CalcMinAndMax(float &min, float &max, BOOL &fThousands,
		   float *pData, int nVals)
{
  min = max = 0.0;
  for(int i=0; i<nVals; i++) {
    if(pData[i] > max)
	    max = pData[i];
    if(pData[i] < min)
	    min = pData[i];
  }
  fThousands = ((fabs(min) > 1000) || (fabs(max) > 1000));
}

//
//  Draw the y-axis divisions and value labels given the
//  pixel coordinates for the upper-left and lower-right
//  corners of the display area, the range of values that
//  the axis spans, the number of divisions to use, and
//  the color for the lines and text.
//
void DrawYAxisInfo(int xLeft, int yTop,
		   int xRight,int yBottom,
		   float minVal, float maxVal, int nDivs,
		   int lineColor, int textColor)
{
  settextjustify(LEFT_TEXT, CENTER_TEXT);
  settextstyle(TRIPLEX_FONT, HORIZ_DIR, 2);
  setlinestyle(DOTTED_LINE, 0, NORM_WIDTH);

  float range = maxVal - minVal;
  float dy = range / nDivs;
  int dyPix = (yBottom - yTop) / nDivs;

  // Calculate # of decimal places to show
  int ndigits;
  if(fabs(dy) < 10/nDivs)
	  ndigits = 2;
  else if(fabs(dy) < 100/nDivs)
	  ndigits = 1;
  else
	  ndigits = 0;

  for(int i=0; i<=nDivs; i++) {

    // Pixel value for line and text output
    int yPix = yTop + i*dyPix;

    // Don't want a dotted line on the edges!
    if((i != 0) && (i != nDivs)) {
	    setcolor(lineColor);
	    line(xLeft, yPix, xRight, yPix);
    }

    // Draw the label to the left of the graph
    float label = maxVal - i*dy;
    char buf[24];
    ostrstream os(buf, 24);
    // Always use fixed point
    os.setf(ios::fixed, ios::floatfield);
    // Display the decimal point, sometimes
    if(ndigits) {
	    os.setf(ios::showpoint);
	    os << setprecision(ndigits);
    }
    // Display ndigits after the decimal
    os << setprecision(ndigits) << label << ends;
    setcolor(textColor);
    outtextxy(xLeft - textwidth(buf), yPix, buf);
  }

  // Draw the 0 line if it's in the display area
  if((minVal < 0) && (maxVal > 0)) {
    int pixRange = yBottom - yTop;
    int yPix = yTop + (int)(maxVal/range * pixRange);
    setlinestyle(SOLID_LINE, 0, THICK_WIDTH);
    setcolor(lineColor);
    line(xLeft, yPix, xRight, yPix);
  }
}

//
//  Draw the x-axis labels given the left and right pixel
//  coordinates, the number of values, and the labels.
//
void DrawXAxisLabels(int xLeft, int xRight, int yBottom,
		   int nVals, char pLabels[][maxLabelLen],
		   int textColor)
{
  settextstyle(TRIPLEX_FONT, HORIZ_DIR, 1);
  settextjustify(CENTER_TEXT, TOP_TEXT);
  setcolor(textColor);

  // The width of each x division, in pixels
  int dxPix = (xRight - xLeft) / nVals;

  for(int i=0; i<nVals; i++) {

    // Truncate each label if it's too long
    // Use a copy -- don't modify the original
    char copy[maxLabelLen];
    strcpy(copy, pLabels[i]);
    while(textwidth(copy) > dxPix * 0.9)
            copy[strlen(copy)-1] = '\0';

    // Write the label in the center of this division
    int xMid = xLeft + i*dxPix + dxPix/2;
    outtextxy(xMid, yBottom, copy);
  }
}

//
//	Draw the titles.
//
void DrawTitles(char *pMainTitle, char *pxAxisTitle,
		char *pyAxisTitle,
		int xLeft, int yTop,
		int xRight, int yBottom,
		int textColor)
{
  // Use this to make string copies that we can modify
  char copy[maxTitleLen];

  int xWidth = xRight - xLeft;
  int xCenter = xLeft + xWidth / 2;
  int yHeight = yBottom - yTop;
  int yCenter = yTop + yHeight / 2;

  setcolor(textColor);

  // Main title
  strcpy(copy, pMainTitle);
  settextjustify(CENTER_TEXT, TOP_TEXT);
  settextstyle(TRIPLEX_FONT, HORIZ_DIR, 7);
  while(textwidth(copy) > getmaxx() - 2)
          copy[strlen(copy)-1] = '\0';
  outtextxy(getmaxx()/2, 5, copy);

  // x-axis title
  strcpy(copy, pxAxisTitle);
  settextjustify(CENTER_TEXT, BOTTOM_TEXT);
  settextstyle(TRIPLEX_FONT, HORIZ_DIR, 2);
  while(textwidth(copy) > xWidth)
          copy[strlen(copy)-1] = '\0';
  outtextxy(xCenter, getmaxy() - 5, copy);

  // y-axis title
  strcpy(copy, pyAxisTitle);
  settextjustify(LEFT_TEXT, CENTER_TEXT);
  settextstyle(TRIPLEX_FONT, VERT_DIR, 3);
  while(textwidth(copy) > yHeight)
          copy[strlen(copy)-1] = '\0';
  outtextxy(5, yCenter, copy);
}

//
//	Draw a square data point centered at the given pixel
//  coordinates in the given color and with sides of the
//  given size.
//
void DrawPoint(int x, int y, int color, int size)
{
  int saveColor = getcolor();
  setcolor(color);

  // cut the size in half for convenience
  size /= 2;
  rectangle(x-size, y-size, x+size, y+size);

  setcolor(saveColor);
}

//
//	Draw each data point and connect them with lines.  The
//  line will be drawn in lineColor, positive points in
//  posPtColor, and negative points in negPtColor.
//
void DrawLines(int xLeft, int yTop, int xRight, int yBottom,
	       float minVal, float maxVal,
	       float *pData, int nVals,
	       int lineColor,
	       int posPtColor, int negPtColor)
{
  setcolor(lineColor);
  setlinestyle(SOLID_LINE, 0, NORM_WIDTH);

  // The width of each x division, in pixels
  int dxPix = (xRight - xLeft) / nVals;

  // Pixel values to keep track of the last point drawn
  int xPrev;
  int yPrev;

  for(int i=0; i<nVals; i++) {
    // The mid point in this division
    int xMid = xLeft + i*dxPix + dxPix/2;

    // The y coordinate for this point
    float yPct = (maxVal - pData[i]) / (maxVal-minVal);
    int yPix = yTop + (int)(yPct * (yBottom - yTop));

    // Plot the point in the appropriate color
    DrawPoint(xMid, yPix,
		      (pData[i] >= 0) ? posPtColor : negPtColor);

    // Draw a line for all points except the first
    if(i > 0)
	    line(xPrev, yPrev, xMid, yPix);

    // So we'll know where to draw from next time
    xPrev = xMid;
    yPrev = yPix;
  }
}

//
//	Draw a bar chart, complete with bars, axes, and titles.
//
void DrawLineChart(char *pMainTitle, char *pxAxisTitle,
	     char *pyAxisTitle,
	     char  pLabels[][maxLabelLen],
	     float *pData, int nVals)
{
  float minVal;
  float maxVal;
  BOOL fThousands;	  // true if abs(any data) > 1000

  //
  //  Find the minimum and maximum values.  Then, if the
  //  any of the data is in thousands, scale it down.
  //
  CalcMinAndMax(minVal, maxVal, fThousands, pData, nVals);
  if(fThousands) {
	  for(int i=0; i<nVals; i++)
		  pData[i] /= 1000.0;
	  minVal /= 1000.0;
	  maxVal /= 1000.0;
  }

  //
  //      Scale the minimum and maximum values so there's some
  //  space at the top and bottom of the graph.
  //
  const float minScaleVal = 0.1;
  const float maxScaleVal = 0.1;
  if(minVal > 0)
	  minVal *= (1.0 - minScaleVal);
  else
	  minVal *= (1.0 + minScaleVal);

  if(maxVal > 0)
	  maxVal *= (1.0 + maxScaleVal);
  else
	  maxVal *= (1.0 - maxScaleVal);

  //
  //  Draw the rectangle around the charting area, using
  //  cyan as the color.
  //
  const int xmax = getmaxx();
  const int ymax = getmaxy();
  const float xLeft   = 0.20 * xmax;
  const float xRight  = 0.95 * xmax;
  const float yTop    = 0.20 * ymax;
  const float yBottom = 0.85 * ymax;

  setcolor(CYAN);
  rectangle(xLeft, yTop, xRight, yBottom);
  setcolor(WHITE);

  // If using scaled data, say so!
  if(fThousands) {
          const char *pThous = "In 1,000's";
	  settextjustify(LEFT_TEXT, BOTTOM_TEXT);
	  outtextxy(xLeft, yTop-3, pThous);
  }

  // Draw the y axis divisions and numerical labels
  const int nYDivs = 5;
  DrawYAxisInfo(xLeft, yTop, xRight, yBottom,
			    minVal, maxVal, nYDivs,
			    CYAN);

  // Draw the x axis labels
  DrawXAxisLabels(xLeft, xRight, yBottom, nVals, pLabels);

  // Draw the lines
  DrawLines(xLeft, yTop, xRight, yBottom, minVal, maxVal,
		    pData, nVals);

  // Draw the titles
  DrawTitles(pMainTitle, pxAxisTitle, pyAxisTitle,
		     xLeft, yTop, xRight, yBottom);
}

void main()
{
  char mainTitle[maxTitleLen];
  char xAxisTitle[maxTitleLen];
  char yAxisTitle[maxTitleLen];
  char labels[maxDataValues][maxLabelLen];
  float data[maxDataValues];

  //  Get the title and data values
  int nVals = GetGraphInfo(mainTitle, xAxisTitle,
			   yAxisTitle, labels, data);
  if(nVals == 0) {
          cout << "No data entered!" << endl;
	  exit(1);
  }

  InitGraphics("C:\\BC3\\BGI");

  DrawLineChart(mainTitle, xAxisTitle, yAxisTitle,
	    labels, data, nVals);

  PromptAndWait("Hit any key to continue...");
  closegraph();
}
