// LIST-MOD.CPP
// Contains the implementation of the list module code.

#include <iostream.h>
#include <string.h>
#include "list-mod.h"

//===============================================
void list_type::Initialize_List(void)
{
  head = NULL;
  tail = NULL;
}

//===============================================
void list_type::Add_to_Tail(char name_to_add[])
// Adds new data to the tail of the list.
{
  list_element *next_entry;   // Points to next entry in list

  // Allocate next entry in list.
  if (head == NULL) { // If list is now empty...
     next_entry = tail = head = new list_element;
     next_entry->next = NULL;
     next_entry->prev = NULL;
  } // if
  else { // List is not empty.
     next_entry = new list_element;
     // Set previous pointer to point at what
     // was the last entry.
     next_entry->prev = tail;
     // Set the previous entry's next pointer to
     // point at the new entry.
     tail->next = next_entry;
     // And set tail to point the new entry.
     tail = next_entry;
     // And the last entry does not point
     // anywhere so set it to NULL.
     next_entry->next = NULL;
  } // else

  strcpy( next_entry->name, name_to_add );
} // Add_to_Tail

//===============================================
void list_type::Display_List(int Direction)
// Displays the content of the list in either
// the FORWARD or the REVERSE direction.
{
  list_element *an_entry; // Temporary pointer to entries.

  switch (Direction) {
    case FORWARD: {
      an_entry = head;
      while (an_entry != NULL) {
	cout << an_entry->name << endl;
      an_entry = an_entry->next;
      } // while
      break;
    } // case
    case REVERSE: {
      an_entry = tail;
      while (an_entry != NULL) {
	cout << an_entry->name << endl;
	an_entry = an_entry->prev;
      } // while
    } // case
    default: break;
  } // switch
} // Display_List

//===============================================
void list_type::Destroy_List(void)
// Frees up all memory used the list.
{
  list_element *next_entry;
  // Delete all of the entries and free
  // up their memory allocations.  This code
  // starts at the end of the list and removes
  // each entry, one by one, until reaching
  // the beginning of the list (where
  // next_entry->prev = NULL).
  while (tail != NULL) {
    next_entry = tail->prev;
    delete tail;
    tail = next_entry;
  } // while
} // Destroy_List


//===============================================
void list_type::Save_to_Disk(fstream &outfile)
{
  list_element *an_entry; // Temporary pointer to entries.

  an_entry = head;
  while (an_entry != NULL) {
    outfile.write( (char *) an_entry->name, MAXNAMESIZE );
    an_entry = an_entry->next;
  }
  outfile.write( (char *) "ENDOFFILE", 10 );
}


//===============================================
void list_type::Open_file(fstream &infile)
{
  char temp_name[MAXNAMESIZE];
  int eof_flag=0;

  do {
    infile.read( (char *) temp_name, MAXNAMESIZE );

    eof_flag = strcmp( temp_name, "ENDOFFILE" ) == 0;

    if (!eof_flag) Add_to_Tail( temp_name );
  } while ( !eof_flag);
}
